d.order <- function(n, r, distfun, densityfun) {
  f <- function(x) NULL
  con <- exp(lgamma(n + 1) - lgamma(r) - lgamma(n - r + 1))
  c1 <- call(substitute(distfun), as.name("x"))
  c2 <- call("^", call("-", 1, c1), n - r)
  c3 <- call("^", c1, r - 1)
  c4 <- call("*", con, call("*", c2, call("*", c3,
             call(substitute(densityfun), as.name("x")))))
  f[[length(f)]] <- c4
  f
}

make.integrand <- function(n, r, cdf, pdf) {
  cdf <- substitute(cdf)
  if(is.name(cdf)) cdf <- call(cdf, as.name("x"))
  pdf <- substitute(pdf)
  if(is.name(pdf)) pdf <- call(pdf, as.name("x"))
  con <- round(exp(lgamma(n+1) - lgamma(r) - lgamma(n-r+1)))
  substitute(function(x) {
               P <- cdf
               K * x * P^r1 * (1 - P)^nr * pdf
             }, list(K = con, cdf = cdf, r1 = r - 1,
                     nr = n - r, pdf = pdf))
}

E.order <- function(n, cdf, pdf, lower = -Inf, upper = Inf, ...)
{
  Ex <- numeric(n)
  for(i in 1:n) {
    f <- eval(call("make.integrand", n, i,
                   substitute(cdf), substitute(pdf)))
    Ex[i] <- integrate(f, lower, upper, ...)[[1]]
  }
  Ex
}

# another approach

make.integrand <- function(n, r, cdf, pdf, ...) {
  con <- round(exp(lgamma(n+1) - lgamma(r) - lgamma(n-r+1)))
  f <- substitute(function(x, cdf, pdf) {
                  P <- cdf(x)
                  K * x * P^r1 * (1 - P)^nr * pdf(x)
                  }, list(K = con, r1 = r - 1, nr = n - r))
  dots <- list(...)
  if(length(dots)) {
    cdf[names(dots)] <- unlist(dots)
    pdf[names(dots)] <- unlist(dots)
  }
  f$cdf <- cdf; f$pdf <- pdf
  f
}
E.order <- function(n, cdf, pdf, lower = -Inf, upper = Inf, ...)
{
  Ex <- numeric(n)
  for(i in 1:n) {
    f <- make.integrand(n, i, cdf, pdf, ...)
    Ex[i] <- integrate(f, lower = lower, upper = upper)[[1]]
  }
  Ex
}
